#include <16F88.h>
#device *=16, ADC=10
#fuses  HS, NOMCLR, NOWDT, NOPROTECT, NOPUT, NOBROWNOUT, NOLVP, NOWRT
#use    delay(clock=20000000)
#use    rs232(baud=19200, xmit=PIN_B5, rcv=PIN_B2, BITS=8, PARITY=N)
#use    fast_io(A)
#use    fast_io(B)

/*                   PIC16F88                      */
/*               +-------u-------+                 */
/*            <--|RA2         RA1|<-- (HRM)        */
/*            <--|RA3         RA0|-->              */
/*            <--|RA4         RA7|<-- CLKI         */
/*            -->|RA5         RA6|--> CLKO         */
/*            ===|VSS         VDD|===              */
/*            <--|RB0         RB7|-->              */
/*            <--|RB1    (AN5)RB6|<-- ECG          */
/*         RX -->|RB2         RB5|--> TX           */
/*            <--|RB3         RB4|-->              */
/*               +---------------+                 */
/* RS232C_Command Cmd Data                         */
/* NOP     00H  --- Sampling Stop                  */
/* Start   5XH  --- Sampling Start 50Hz filter     */
/* Start   6XH  --- Sampling Start 60Hz filter     */
/* Start   X1H  --- Sampling ECG                   */
/* Start   X2H  --- Sampling HRT                   */
/* Start   X3H  --- Sampling ECG + HRT             */

#define       nFIFO 32
int8          cmd, cnt, ptr, st;
signed int16 Din, sum, FIFO[nFIFO];
unsigned int16 Dout, dt;

void InitializeFilter(void)
{  int8 i;
   cmd=0; cnt=0; ptr=0; sum=0;
   dt=0xF3CA;//50Hz
   st=16;//10msec*50Hz*32sample
   for( i=0; i<nFIFO; i++ ) { FIFO[i]=0; }
}

#int_rda
void receive_byte(void)
{  cmd=getc();
   switch( cmd )
   {  case 0x51:
         dt=0xF3CA;
         st=16;
         cmd=1;
         break;
      case 0x52:
         dt=0xF3CA;
         st=16;
         cmd=2;
         break;
      case 0x53:
         dt=0xF3CA;
         st=16;
         cmd=3;
         break;
      case 0x61:
         dt=0xF5D3;// 1000/(60Hz*32)msec  65535-(0.0005208*20000000/4)
         st=19;// 10msec*60Hz*32sample
         cmd=1;
         break;
      case 0x62:
         dt=0xF5D3;
         st=19;
         cmd=2;
         break;
      case 0x63:
         dt=0xF5D3;
         st=19;
         cmd=3;
         break;
      default: cmd=0; break;
   }
}

#int_timer1
void timer_event(void)
{  set_timer1(dt);
   if( cmd!=0 )
   {  Din=read_adc();
      sum=sum-FIFO[ptr]+Din;
      FIFO[ptr]=Din;
      ptr++; if( ptr==nFIFO ) ptr=0;
      cnt++;
      switch( cnt )
      {  case 1:
            Dout=0xFFFF;
            putc(make8(Dout,0));
            enable_interrupts( INT_TBE );
            break;
         case 6:
            if((cmd & 0x01) != 0)
            {  Dout=(unsigned int16)sum >> 5;
               putc(make8(Dout,0));
               enable_interrupts( INT_TBE );
            }
            else
            {  Dout=0x0000;
               putc(make8(Dout,0));
               enable_interrupts( INT_TBE );
            }
            break;
         case 11:
            set_adc_channel(1);
            delay_us(18);
            if((cmd & 0x02) != 0)
            {  Dout=read_adc();
               putc(make8(Dout,0));
            }
            else
            {  Dout=0x0000;
               putc(make8(Dout,0));
            }
            set_adc_channel(5);
            enable_interrupts( INT_TBE );
            break;
         default: break;
      }
      if(cnt == st) cnt=0;
   }
}

#int_tbe
void send_2nd_byte(void)
{  putc(make8(Dout,1));
   disable_interrupts( INT_TBE );
}

void main()
{  port_b_pullups(FALSE);           /* No Pullup  */
   output_B(0b00100000);
   set_tris_B(0b01000100);          /* Port_B Set 0=out 1=in */
   output_A(0b00000000);
   set_tris_A(0b10100010);          /* Port_A Set */
   setup_adc( ADC_CLOCK_DIV_32 );   /* @ 20MHz  */
   setup_adc_ports(sAN1|sAN5|VSS_VDD);
   set_adc_channel(5);
   InitializeFilter();
   setup_timer_1(T1_INTERNAL | T1_DIV_BY_1);
   set_timer1(dt);
   enable_interrupts(INT_TIMER1);
   enable_interrupts(INT_RDA);
   enable_interrupts(GLOBAL);
   while(TRUE) {      }
}

